#include "stdafx.h"
#include "driver.h"



/*++
===============================================================================
Function AES_Init
This function is called by Device Manager to initialize the streams interface
in response to ActivateDevice. We passed ActivateDevice a pointer to our
device context, but must read it out of the registry as "ClientInfo".
Parameters
Context [in] Port identifier. This value points to a string
containing the registry key within
HKEY_LOCAL_MACHINE\Drivers\Active\xx.
lpvBusContext [in] Not used.
Return Values
Returns a pointer to the context used as a device handle into the entry points
AES_Open, AES_Close, AES_Read, AES_Write, etc.
Remarks
This function is called at device load time in order to perform any
initialization. Typically, this function does as little as possible. This
function postpones memory allocation and device power until an application
calls the driver's AES_Open function.
===============================================================================
*/
DWORD
DEV_Init(
		 LPCTSTR lpszContext, LPCVOID lpvBusContext
		 )
{

	PUSBFPS_CONTEXT pUsbFps = NULL;
	// registry value that need to be read:
	REG_VALUE_DESCR usbKeyValues[] =
	{
		(TEXT("ClientInfo")), REG_DWORD, sizeof(DWORD), (PBYTE)(&pUsbFps),
		NULL, 0, 0, NULL
	};

	DEBUGMSG(ZONE_AES_INIT, (TEXT(">DEV_Init(%p)\n"), lpszContext));
	// retrieve the pointer to the device context from the registry...
	GetSetKeyValues( lpszContext,
		&usbKeyValues[0],
		GET,
		FALSE );
	DEBUGMSG(ZONE_AES_INIT, (TEXT("<DEV_Init:0x%x\n"), pUsbFps ));

	if (pUsbFps)
	{
		AddLog("DEV_Init Succsessfull");
	}
	else
	{
		AddLog("DEV_Init Unsucsessfull");
	}
	return((DWORD) pUsbFps);
}
/*++
===============================================================================
Function AES_Deinit
This function deinitializes
the stream interface.
Parameters
pUsbFps [in] Pointer to the device context information
Return Values
None.
Remarks
This function is called by the DeregisterDevice function, which is exposed
to applications.
===============================================================================
*/
DWORD
DEV_Deinit(
		   DWORD dwContext     // future: pointer to the per disk structure
		   )
{
	AddLog("DEV_Deinit");
	DEBUGMSG(ZONE_AES_INIT, (TEXT(">AES_Deinit\n")));
	DEBUGMSG(ZONE_AES_INIT, (TEXT("<AES_Deinit\n")));
	return TRUE;
}
/*++
===============================================================================
Function AES_Open
This function initializes the stream interface for the USB driver.
Parameters
pUsbFps [in] Pointer to a handle returned by the AES_Init function.
AccessCode [in] Specifies the access level requested by the caller, such
as shared read or read/write.
ShareMode [in] Specifies the sharing mode that the calling application
passed to the CreateFile function.
Return Values
Returns a handle representing the USB Finger Print Sensor context. If the
device cannot be opened, this function returns NULL.
Remarks
The user calls this function to open the device. The HANDLE returned
must be used by the application in all subsequent calls to the driver.
This function starts the thread that handles the serial events. This function
is exported to applications through CreateFile.
===============================================================================
*/
PUSBFPS_CONTEXT
DEV_Open(PUSBFPS_CONTEXT pUsbFps,
		 DWORD AccessCode,
		 DWORD ShareMode)
{
	AddLog("DEV_Open");
	BOOL bRc = TRUE; // validity flag	
	DEBUGMSG(ZONE_AES_INIT,	(TEXT(">AES_Open(0x%x, 0x%x, 0x%x)\n"),	pUsbFps, AccessCode, ShareMode));
	if ( VALID_CONTEXT( pUsbFps ) )
	{
		EnterCriticalSection(&pUsbFps->Control.Lock);
		EnterCriticalSection(&pUsbFps->BulkIn.Lock);
		EnterCriticalSection(&pUsbFps->BulkOut.Lock);
		if (!pUsbFps->Flags.Open &&	!pUsbFps->Flags.UnloadPending )
		{
			// say the file is opened...
			pUsbFps->Flags.Open = TRUE;
			// set close event as nonsignaled...
			ResetEvent(pUsbFps->hCloseEvent );
		}
		else
		{
			DEBUGMSG( ZONE_ERR,(TEXT("AES_Open: ERROR_ACCESS_DENIED\n")));
			SetLastError(ERROR_ACCESS_DENIED);
			bRc = FALSE;
		}

		LeaveCriticalSection(&pUsbFps->Control.Lock);
		LeaveCriticalSection(&pUsbFps->BulkIn.Lock);
		LeaveCriticalSection(&pUsbFps->BulkOut.Lock);
	}
	else
	{
		DEBUGMSG(ZONE_ERR, (TEXT("AES_Open: ERROR_FILE_NOT_FOUND\n")));
		SetLastError(ERROR_FILE_NOT_FOUND);
		bRc = FALSE;
	}
	DEBUGMSG(ZONE_AES_INIT, (TEXT("<AES_Open:%d\n"), bRc ));
	return (bRc ? pUsbFps : NULL);
}
/*++
===============================================================================
Function AES_Close
This function closes the stream driver. It is called after an application
calls the CloseHandle function.
Parameters
pUsbFps [in] Context pointer returned from the AES_Open function.
Return Values
TRUE indicates success. FALSE indicates failure.
Remarks
The Device Manager calls this routine to close the device.
===============================================================================
*/

DWORD
DEV_Close(
		  PUSBFPS_CONTEXT pUsbFps
		  )
{
	AddLog("DEV_Close");
	DEBUGMSG(ZONE_AES_INIT,(TEXT("AES_Close(0x%x)\n"),pUsbFps));
	if ( VALID_CONTEXT( pUsbFps ) )
	{
		EnterCriticalSection(&pUsbFps->Control.Lock);
		EnterCriticalSection(&pUsbFps->BulkIn.Lock);
		EnterCriticalSection(&pUsbFps->BulkOut.Lock);

		// say the file is closed...
		pUsbFps->Flags.Open = FALSE;
		// halt any pending transfers...
		SetEvent(pUsbFps->BulkOut.hEvent);

		LeaveCriticalSection(&pUsbFps->Control.Lock);
		LeaveCriticalSection(&pUsbFps->BulkIn.Lock);
		LeaveCriticalSection(&pUsbFps->BulkOut.Lock);

		// Note: any waiters are run as soon as we signal this event...
		return SetEvent( pUsbFps->hCloseEvent );
	}
	else
	{
		DEBUGMSG(ZONE_ERR, (TEXT("AES_Close: ERROR_INVALID_HANDLE\n")));
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
	}
}
/*++
===============================================================================
Function AES_Read
This function enables an application to receive characters from the USB device.
Parameters
pUsbFps [in] Pointer to a handle returned by the AES_Open function.
pBuffer [in] Pointer to valid memory.
dwBufferLength [in] Specifies the size, in bytes, of pTargetBuffer.
Return Values
The number of bytes read indicates success. A value of 1
indicates failure.
Remarks
AES_Read sets the buffer and buffer length requested by the caller. It writes
the number of bytes transacted and returns the status of the call.
This function is exported to users through the ReadFile function.
This function must obey timeout
values set for the serial port.
===============================================================================
*/
DWORD	
DEV_Read(PUSBFPS_CONTEXT pUsbFps, PUCHAR pBuffer, DWORD dwBufferLength)
{
	AddLog("ReadUsb Begin");

	DWORD dwTotalBytesTransferred=0;		

	// is the device currently available?
	if ( ACCEPT_IO( pUsbFps ) && pBuffer && dwBufferLength)
	{
		DWORD dwErr = ERROR_SUCCESS;
		DWORD dwUsbErr = USB_NO_ERROR;			

		// is there a buffer for transfer?
		while (dwBufferLength )
		{
			DWORD BytesToRecieveOnce = min(dwBufferLength, 8*1024);

			EnterCriticalSection( &pUsbFps->BulkIn.Lock );
			DWORD dwBytesTransferred=0;			

			dwErr = IssueBulkTransfer(pUsbFps->lpUsbFuncs,
				pUsbFps->BulkIn.hPipe,
				DefaultTransferComplete,
				pUsbFps->BulkIn.hEvent,
				(USB_IN_TRANSFER | USB_SHORT_TRANSFER_OK),
				pBuffer, 0,
				BytesToRecieveOnce,
				&dwBytesTransferred,
				pUsbFps->BulkIn.Timeout,
				&dwUsbErr );

			LeaveCriticalSection( &pUsbFps->BulkIn.Lock );
			// was an error detected?
			if ((ERROR_SUCCESS != dwErr) || (USB_NO_ERROR != dwUsbErr))
			{
				AddLogMessage3("Error During ReadBulk, dwErr=%d, dwUsbErr=%d, timeout=%d",dwErr,dwUsbErr, pUsbFps->BulkIn.Timeout);
				// the device was probably unplugged...
				if (ERROR_GEN_FAILURE == dwErr)
				{
					// return failure...
					return((DWORD) -1);
				}
			}

			if (!dwBytesTransferred) break;

			dwBufferLength-=dwBytesTransferred;
			pBuffer+=dwBytesTransferred;
			dwTotalBytesTransferred+=dwBytesTransferred;
		}		
	}
	else
	{
		AddLog("ReadUsb: ERROR_INVALID_HANDLE");
		DEBUGMSG( ZONE_ERR,(TEXT("ReadUsb: ERROR_INVALID_HANDLE\n")));
		SetLastError(ERROR_INVALID_HANDLE);
	}

	AddLogMessage1("ReadUsb End, %d bytes readed",dwTotalBytesTransferred);	
	return dwTotalBytesTransferred;
}
/*++
===============================================================================
Function AES_Write
This function enables an application to transmit bytes via the USB bulk out
pipe.
Parameters
pUsbFps [in] Handle returned by the AES_Open function.
pBuffer [in] Pointer to the bytes to be written.
dwBufferLength [in] Specifies the number of bytes to be written.
Return Values
The number of bytes written indicates success. A value of 1
indicates failure.
Remarks
This function is exported to users through the WriteFile function. This
function must obey flowcontrol
and timeout
values.
===============================================================================
*/
DWORD	
DEV_Write(PUSBFPS_CONTEXT pUsbFps, PUCHAR pBuffer, DWORD dwBufferLength)
{
	AddLog("WriteUsb Begin");

	DWORD dwTotalTransferred = 0 ;
	DWORD dwErr = ERROR_SUCCESS;
	DWORD dwUsbErr = USB_NO_ERROR;		
	DEBUGMSG(ZONE_AES_WRITE, (TEXT(">WriteUsb(0x%x, %d)\n"),pBuffer, dwBufferLength));
	// is the device currently available?
	if ( ACCEPT_IO( pUsbFps ) &&pBuffer && dwBufferLength)
	{
		while (dwBufferLength)
		{			
			DWORD BytesToWriteOnce = min(dwBufferLength, 8*1024);

			EnterCriticalSection( &pUsbFps->BulkOut.Lock );
			DWORD dwBytesTransferred = 0 ;
			dwErr = IssueBulkTransfer(pUsbFps->lpUsbFuncs,
				pUsbFps->BulkOut.hPipe,
				DefaultTransferComplete,
				pUsbFps->BulkOut.hEvent,
				(USB_OUT_TRANSFER),
				pBuffer, 0,
				BytesToWriteOnce,
				&dwBytesTransferred,
				pUsbFps->BulkOut.Timeout,
				&dwUsbErr );

			LeaveCriticalSection( &pUsbFps->BulkOut.Lock );

			if ((ERROR_SUCCESS != dwErr) || (USB_NO_ERROR != dwUsbErr))
			{
				AddLogMessage3("Error During WriteBulk, dwErr=%d, dwUsbErr=%d, timeout=%d",dwErr,dwUsbErr, pUsbFps->BulkOut.Timeout);
				DEBUGMSG( ZONE_ERR, (TEXT("IssueBulkTransfer (Write) error:%d, 0x%x\n"), dwErr, dwUsbErr));				
				return -1;
			}						
			if (!dwBytesTransferred) break;

			pBuffer += dwBytesTransferred;
			dwTotalTransferred += dwBytesTransferred;			
			dwBufferLength -=dwBytesTransferred;		
		}
	}
	else
	{
		AddLog("WriteUsb: ERROR_INVALID_HANDLE");
		DEBUGMSG( ZONE_ERR,(TEXT("WriteUsb: ERROR_INVALID_HANDLE\n")));
		dwTotalTransferred = (DWORD)-1;
		SetLastError(ERROR_INVALID_HANDLE);
	}

	AddLogMessage1("WriteUsb End, %d bytes written",dwTotalTransferred);	

	return dwTotalTransferred;
}
//===============================================================================
//
//===============================================================================
DWORD	
DEV_Seek(DWORD Handle, long lDistance, DWORD dwMoveMethod)
{
	AddLog("DEV_Seek");
	return 0;
}
/*++
===============================================================================
Function AES_IOControl
This function implements the USB port's I/O control routine.
Parameters
pUsbFps [in] Specifies a value returned from a call to the
AES_Open function.
dwCode [in] Specifies an I/O control code to be performed.
pInBuf [in] Pointer to data input to the device.
dwLenIn [in] Specifies the number of bytes being passed in.
pOutBuf [out] Pointer to data output from the driver.
dwLenOut [out] Specifies the maximum number of bytes to receive
from the driver.
pdwActualOut [out] Pointer to the actual number of bytes received
from the driver.
Return Values
TRUE indicates success. FALSE indicates failure.
Remarks
This function is exported by a device driver through the DeviceIoControl()
function.
===============================================================================
*/
DWORD	
DEV_IOControl(PUSBFPS_CONTEXT pUsbFps,
			  DWORD dwCode,
			  PBYTE pBufIn,
			  DWORD dwLenIn,
			  PBYTE pBufOut,
			  DWORD dwLenOut,
			  PDWORD pdwActualOut)
{
	AddLog("DEV_IOControl");	
	DEBUGMSG(ZONE_AES_IOCTL, (TEXT(">AES_IOControl(0x%x, 0x%x, %d, 0x%x)\n"), dwCode, pBufIn, dwLenIn, pBufOut, dwLenOut ));
	// is the device currently available?
	if ( ACCEPT_IO( pUsbFps ) )
	{
		switch ( dwCode )
		{
		case IOCTL_CONTROL_PIPE_VENDOR:
			if (dwLenIn!=sizeof(_ControlRequest))
			{
				SetLastError(ERROR_INVALID_PARAMETER);
				return FALSE;
			}

			return ControlIO(pUsbFps, (_ControlRequest *)pBufIn, pBufOut,dwLenOut, pdwActualOut);			

		case IOCTL_CONTROL_SET_READ_BULK_TIMEOUT_MS:					
			if ((NULL != pBufIn) && (dwLenIn >= sizeof(DWORD)))
			{				
				EnterCriticalSection(&pUsbFps->BulkIn.Lock);				
				memcpy(&pUsbFps->BulkIn.Timeout,pBufIn,sizeof(DWORD));
				LeaveCriticalSection(&pUsbFps->BulkIn.Lock);

				AddLogMessage1("Set Bulk read Timeout %d",pUsbFps->BulkIn.Timeout);
				return TRUE;
			}
			else
			{				
				SetLastError(ERROR_INVALID_PARAMETER);
				return FALSE;
			}			
		case IOCTL_CONTROL_SET_WRITE_BULK_TIMEOUT_MS:					
			if ((NULL != pBufIn) && (dwLenIn >= sizeof(DWORD)))
			{
				EnterCriticalSection(&pUsbFps->BulkOut.Lock);				
				memcpy(&pUsbFps->BulkOut.Timeout,pBufIn,sizeof(DWORD));
				LeaveCriticalSection(&pUsbFps->BulkOut.Lock);
				AddLogMessage1("Set Bulk write Timeout %d",pUsbFps->BulkOut.Timeout);
				return TRUE;
			}
			else
			{				
				SetLastError(ERROR_INVALID_PARAMETER);
				return FALSE;
			}	
		case IOCTL_CONTROL_SET_CONTROL_TIMEOUT_MS:					
			if ((NULL != pBufIn) && (dwLenIn >= sizeof(DWORD)))
			{
				EnterCriticalSection(&pUsbFps->Control.Lock);				
				memcpy(&pUsbFps->Control.Timeout,pBufIn,sizeof(DWORD));
				LeaveCriticalSection(&pUsbFps->Control.Lock);
				AddLogMessage1("Set control Timeout %d",pUsbFps->Control.Timeout);
				return TRUE;
			}
			else
			{				
				SetLastError(ERROR_INVALID_PARAMETER);
				return FALSE;
			}	
		case IOCTL_CONTROL_GET_READ_BULK_TIMEOUT_MS:					
			if ((NULL != pBufIn) && (dwLenOut >= sizeof(DWORD)))
			{
				EnterCriticalSection(&pUsbFps->BulkIn.Lock);				
				memcpy(pBufOut,&pUsbFps->BulkIn.Timeout,sizeof(DWORD));
				LeaveCriticalSection(&pUsbFps->BulkIn.Lock);
				AddLogMessage1("Get Bulk read Timeout %d",pUsbFps->BulkIn.Timeout);
				return TRUE;
			}
			else
			{				
				SetLastError(ERROR_INVALID_PARAMETER);
				return FALSE;
			}			
		case IOCTL_CONTROL_GET_WRITE_BULK_TIMEOUT_MS:					
			if ((NULL != pBufIn) && (dwLenOut >= sizeof(DWORD)))
			{
				EnterCriticalSection(&pUsbFps->BulkOut.Lock);				
				memcpy(pBufOut,&pUsbFps->BulkOut.Timeout,sizeof(DWORD));
				LeaveCriticalSection(&pUsbFps->BulkOut.Lock);
				AddLogMessage1("Get Bulk write Timeout %d",pUsbFps->BulkOut.Timeout);
				return TRUE;
			}
			else
			{				
				SetLastError(ERROR_INVALID_PARAMETER);
				return FALSE;
			}	
		case IOCTL_CONTROL_GET_CONTROL_TIMEOUT_MS:					
			if ((NULL != pBufIn) && (dwLenOut >= sizeof(DWORD)))
			{
				EnterCriticalSection(&pUsbFps->Control.Lock);				
				memcpy(pBufOut,&pUsbFps->Control.Timeout,sizeof(DWORD));
				LeaveCriticalSection(&pUsbFps->Control.Lock);
				AddLogMessage1("Get control Timeout %d",pUsbFps->Control.Timeout);
				return TRUE;
			}
			else
			{				
				SetLastError(ERROR_INVALID_PARAMETER);
				return FALSE;
			}	
		case IOCTL_CONTROL_SUSPEND_DEVICE:
			return pUsbFps->lpUsbFuncs->lpSuspendDevice(pUsbFps->hUsbDevice,0);			

		case IOCTL_CONTROL_UNSUSPEND_DEVICE:
			return pUsbFps->lpUsbFuncs->lpResumeDevice(pUsbFps->hUsbDevice,0);

		case IOCTL_CONTROL_RESET_CONTROL_PIPE:
			return pUsbFps->lpUsbFuncs->lpResetDefaultPipe(pUsbFps->hUsbDevice);		

		case IOCTL_CONTROL_RESET_BULK_IN_PIPE:
			return pUsbFps->lpUsbFuncs->lpResetPipe(pUsbFps->BulkIn.hPipe);			
			
		case IOCTL_CONTROL_RESET_BULK_OUT_PIPE:
			return pUsbFps->lpUsbFuncs->lpResetPipe(pUsbFps->BulkOut.hPipe);			
	
		case IOCTL_CONTROL_ABORT_BULK_IN_PIPE:
			return pUsbFps->lpUsbFuncs->lpAbortPipeTransfers(pUsbFps->BulkIn.hPipe,0);

		case IOCTL_CONTROL_ABORT_BULK_OUT_PIPE:
			return pUsbFps->lpUsbFuncs->lpAbortPipeTransfers(pUsbFps->BulkOut.hPipe,0);
			
		case IOCTL_CONTROL_GET_PARAMS:
			{
				if (dwLenOut!=sizeof(DEVICE_INITIAL_INFO)) return FALSE;				
				S_DEVICE_INITIAL_INFO * DevInfo = (S_DEVICE_INITIAL_INFO *)pBufOut;
				LPCUSB_DEVICE DevInfoInternal = pUsbFps->lpUsbFuncs->lpGetDeviceInfo(pUsbFps->hUsbDevice);
				switch (DevInfoInternal->Descriptor.idProduct)
				{
				case 0x440 :DevInfo->DeviceID=E14_440_ID; break;
				case 0x140 :DevInfo->DeviceID=E14_140_ID; break;
				case 0x2010 :DevInfo->DeviceID=E2010_ID; break;
				case 0x270 :DevInfo->DeviceID=E270_ID; break;
				case 0x10 :DevInfo->DeviceID=LTR010_ID; break;
				case 0x21 :DevInfo->DeviceID=LTR021_ID; break;
				case 0x154 :DevInfo->DeviceID=E154_ID; break;
				default:DevInfo->DeviceID = INVALID_USB_DEVICE_ID; break;
				}				
				return TRUE;
			}

		default:
			DEBUGMSG(ZONE_ERR,
				(TEXT("AES_IOControl(0x%x) : ERROR_NOT_SUPPORTED\n"),
				dwCode));
			SetLastError(ERROR_NOT_SUPPORTED);
			return FALSE;
			break;
		}
	}
	else
	{
		DEBUGMSG( ZONE_ERR,(TEXT("AES_IOControl: ERROR_INVALID_HANDLE\n")));
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
	}	
	return FALSE;
}
//===============================================================================
//
//===============================================================================
DWORD	
DEV_PowerDown(void)
{
	AddLog("DEV_PowerDown");
	return 0;
}
//===============================================================================
//
//===============================================================================
DWORD	
DEV_PowerUp(void)
{
	AddLog("DEV_PowerUp");
	return 0;
}
