#include "stdafx.h"
#include "driver.h"
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//
/*++
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Module Name:

usbddrv.cpp

Abstract:

This file contains code for the USBD module of the Universal Serial
Bus driver for Windows CE.  The USBD driver is responsible for loading
client drivers, and providing an interface to the USB host controller
to client drivers.

  +--------------+
  | USB Client   |
  |   Driver     |
  +--------------+
        /|\
         |       USBDI Interface
        \|/
  +--------------+
  |  USBD        |
  |  Driver      |
  +--------------+
        /|\
         |       HCD Interface
        \|/
  +--------------+
  |  HCD (OHCD or|
  |  UHCD) driver|
  +--------------+
        /|\
         |       HC Interface
        \|/
  +--------------------+               +------------------------+
  | Host controller HW |               | USB Device (function)  |
  +--------------------+               +------------------------+
        /|\                                        /|\
         |                                          |
         +------------------------------------------+

Functions:

TranslateStringDescr
LoadGenericInterfaceDriver
OpenClientRegistryKey
GetClientRegistryPath
FindInterface
TakeFrameLengthControl
ReleaseFrameLengthControl
SetFrameLength
GetFrameNumber
GetFrameLength
OpenPipe
ResetPipe
AbortPipeTransfers
GetInterface
SetInterface
GetDescriptor
SetDescriptor
SetFeature
ClearFeature
GetStatus
SyncFrame
IssueVendorTransfer
IssueControlTransfer
IssueBulkTransfer
IssueInterruptTransfer
IssueIsochTransfer
CloseTransfer
AbortTransfer
ClosePipe
IsTransferComplete
GetTransferError
RegisterNotificationRoutine
UnRegisterNotificationRoutine
GetTransferStatus
GetIsochResults
GetDeviceInfo
GetUSBDVersion
IsPipeHalted

Notes:

@doc DRIVERS
--*/
#pragma warning (3 : 4100 4101 4705 4706)

#include <windows.h>
#include <netui.h>
#include <windev.h>

#include "usbdi.h"
#include "hcdi.h"

//#include "usbdinc.hpp"


LPCWSTR gcszUsbRegKey = L"Drivers\\USB";
LPCWSTR gcszDriverIDs = L"ClientDrivers";
LPCWSTR gcszLoadClients = L"LoadClients";
LPCWSTR gcszDefaultDriver = L"Default";
LPCWSTR gcszDllName = L"DLL";
LPCWSTR gcszUSBDeviceAttach = L"USBDeviceAttach";
LPCWSTR gcszUSBInstallDriver = L"USBInstallDriver";
#define UnusedParameter(x) x = x

const int gcMaxPipes = 31; // 1 bi (ep0) + 15 in + 15 out
const int gcTransferAllocateChunk = 4;
const int gcEndpointZero = 0;

const int gcMaxDriverString = 100;
const int gcMaxDriverIdSize = 256;
const int gcCancelGranularity = 20;

#ifdef DEBUG
#define DBG_INIT     0x0001
#define DBG_LOADER   0x0002
#define DBG_CONFIG   0x0004
#define DBG_PIPE     0x0008
#define DBG_VENDOR   0x0010
#define DBG_CONTROL  0x0020
#define DBG_BULK     0x0040
#define DBG_INTR     0x0080
#define DBG_ISOCH    0x0100
#define DBG_API      0x0200
#define DBG_WARN     0x4000
#define DBG_ERROR    0x8000
DBGPARAM dpCurSettings = {
    TEXT("USBD"), {
        TEXT("Init"),TEXT("Loader"),TEXT("Config"),TEXT("Pipe"),
        TEXT("Vendor"),TEXT("Control"),TEXT("Bulk"),TEXT("Interrupt"),
        TEXT("Isoch"),TEXT("API"),TEXT("Unused"),TEXT("Unused"),
        TEXT("Unused"),TEXT("Unused"),TEXT("Warning"),TEXT("Error")},
    DBG_INIT | DBG_PIPE | DBG_ERROR
};

// For decoding endpoint descriptor attributes
static const TCHAR *aszTypeStrings[] =
{
    TEXT("Control"),TEXT("Isoch"),TEXT("Bulk"),TEXT("Interrupt")
};

#endif
LPCWSTR gcszUsbConfigureEntry = L"bConfigurationValue";









/*
 * @func   BOOL | TranslateStringDescr | Translate a USB string descriptor into a NULL terminated string.
 * @rdesc  Returns TRUE if lpStringDescr points to a valid USB string descriptor.
 * @comm   If the string to be translated would overflow the supplied buffer,
 *         it is truncated to fit.
 */
extern "C" BOOL
TranslateStringDescr(
    LPCUSB_STRING_DESCRIPTOR lpStringDescr,  // @parm [IN] - Pointer to USB string descriptor
    LPWSTR szString,                         // @parm [OUT]- Buffer to receive string data
    DWORD cchStringLength)                   // @parm [IN] - Size of string buffer (characters)
{
    ASSERT(sizeof(szString[0]) == sizeof(lpStringDescr->bString[0]));

    szString[0] = 0;

    if(lpStringDescr->bDescriptorType != USB_STRING_DESCRIPTOR_TYPE)
        return FALSE;

    UINT cchString = (lpStringDescr->bLength - sizeof(USB_COMMON_DESCRIPTOR)) /
            sizeof(lpStringDescr->bString[0]);
    --cchStringLength; // normalize to actual # of characters we can copy

    if(cchString > cchStringLength)
        cchString = cchStringLength;

    memcpy(szString, &lpStringDescr->bString, cchString * sizeof(szString[0]));

    szString[cchString] = 0;

    return TRUE;
}



/*
 * @func  BOOL | RegisterClientDriverID | Called to register a unique string to identify a client driver.
 * @rdesc Return TRUE if client id was successfully registered, FALSE if an error occurred.
 */
extern "C" BOOL
RegisterClientDriverID(
    LPCWSTR szUniqueDriverId)
{
    BOOL fRet = FALSE;
    DWORD dwGarbage;

    HKEY hkUsb;

    if(RegCreateKeyEx(HKEY_LOCAL_MACHINE, gcszUsbRegKey, 0, NULL, 0, 0, NULL,
            &hkUsb, &dwGarbage) == ERROR_SUCCESS)
    {
        HKEY hkDrivers;

        if(RegCreateKeyEx(hkUsb, gcszDriverIDs, 0, NULL, 0, 0, NULL,
                &hkDrivers, &dwGarbage) == ERROR_SUCCESS)
        {
            HKEY hkThisDriver;

            if(RegCreateKeyEx(hkDrivers, szUniqueDriverId, 0, NULL, 0, 0,
                    NULL, &hkThisDriver, &dwGarbage) == ERROR_SUCCESS)
            {
                fRet = TRUE;

                RegCloseKey(hkThisDriver);
            }
            RegCloseKey(hkDrivers);
        }
        RegCloseKey(hkUsb);
    }

    return fRet;
}

extern "C" BOOL UnRegisterClientDriverID(LPCWSTR szUniqueDriverId)
{
    BOOL fRet = FALSE;

    HKEY hkUsb;

    if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, gcszUsbRegKey, 0, 0, &hkUsb)
            == ERROR_SUCCESS)
    {
        HKEY hkDrivers;

        if(RegOpenKeyEx(hkUsb, gcszDriverIDs, 0, 0, &hkDrivers)
                == ERROR_SUCCESS)
        {
            if(RegDeleteKey(hkDrivers, szUniqueDriverId) == ERROR_SUCCESS)
            {
                fRet = TRUE;
            }
            RegCloseKey(hkDrivers);
        }
        RegCloseKey(hkUsb);
    }

    return fRet;
}

/*
 * @func   HKEY | OpenClientRegistryKey | Opens registry key associated with client driver.
 * @rdesc  Return handle to open key, or NULL if key doesn't exist, or other error occurs.
 * @comm   Client key is created in <f RegisterClientDriverId>.
 * @xref   <f RegisterClientDriverId>
 */
extern "C" HKEY
OpenClientRegistryKey(
    LPCWSTR szUniqueDriverId)  // @parm [IN] - Unique driver Id string
{
    HKEY hkUsb;
    HKEY hkDriverKey = NULL;

    if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, gcszUsbRegKey, 0, 0, &hkUsb)
            == ERROR_SUCCESS)
    {
        HKEY hkDrivers;

        if(RegOpenKeyEx(hkUsb, gcszDriverIDs, 0, 0, &hkDrivers)
                == ERROR_SUCCESS)
        {
            if(RegOpenKeyEx(hkDrivers, szUniqueDriverId, 0, 0, &hkDriverKey)
                    != ERROR_SUCCESS)
            {
                hkDriverKey = NULL;
            }
            RegCloseKey(hkDrivers);
        }
        RegCloseKey(hkUsb);
    }

    return hkDriverKey;
}
/*
 * @func   BOOL | GetClientRegistryPath | reutrn registry path associated with client driver.
 * @rdesc  Return handle to open key, or NULL if key doesn't exist, or other error occurs.
 * @comm   Client key is created in <f RegisterClientDriverId>.
 * @xref   <f RegisterClientDriverId>
 */
extern "C" BOOL
GetClientRegistryPath(LPWSTR szRegistryPath, DWORD dwRegPathUnit, LPCWSTR szUniqueDriverId)
{
    if (szRegistryPath && dwRegPathUnit && szUniqueDriverId) {
        HRESULT rResult;
        if (SUCCEEDED(rResult=StringCchCopy(szRegistryPath,dwRegPathUnit,gcszUsbRegKey)) &&
                SUCCEEDED(rResult=StringCchCat(szRegistryPath,dwRegPathUnit,TEXT("\\"))) &&                
                SUCCEEDED(rResult=StringCchCat(szRegistryPath,dwRegPathUnit,gcszDriverIDs)) &&                
                SUCCEEDED(rResult=StringCchCat(szRegistryPath,dwRegPathUnit,TEXT("\\"))) &&                
                SUCCEEDED(rResult=StringCchCat(szRegistryPath,dwRegPathUnit,szUniqueDriverId)) ) {
            return TRUE;
        }
        else {
            SetLastError(HRESULT_CODE(rResult));
        }
    }
    else
        SetLastError(ERROR_INVALID_PARAMETER);
    return FALSE;
}

void GetSettingString(LPTSTR pszString, DWORD cchString, DWORD dw1, DWORD dw2, DWORD dw3)
{
    LPCTSTR pszFormat = NULL;

    if (dw3 != USB_NO_INFO) {
        ASSERT( (dw1 != USB_NO_INFO) && (dw2 != USB_NO_INFO) );
        pszFormat = TEXT("%u_%u_%u");
    }
    else if (dw2 != USB_NO_INFO) {
        ASSERT(dw1 != USB_NO_INFO);
        pszFormat = TEXT("%u_%u");
    }
    else if (dw1 != USB_NO_INFO) {
        pszFormat = TEXT("%u");
    }

    if (pszFormat == NULL) {
        // if there are no specific settings then return a default setting
        StringCchCopy(pszString, cchString, gcszDefaultDriver);
    }
    else {
        StringCchPrintf(pszString, cchString, pszFormat, dw1, dw2, dw3);
    }
}


/*
 * @func   BOOL | RegisterClientSettings | Called to set up client driver load settings.
 * @rdesc  Returns TRUE if settings were successfully configured, otherwise FALSE.
 * @comm   This function should be called by client drivers in their installation routine.
 *         It sets up appropriate registry keys based on lpDriverSettings so that the
 *         driver will be loaded whenever the device is attached.
 * @xref   <t USB_DRIVER_SETTINGS>  <f UnRegisterClientSettings>
 */
extern "C" BOOL
RegisterClientSettings(
    LPCWSTR szDriverLibFile,                // @parm [IN] - Client driver DLL name
    LPCWSTR szUniqueDriverId,               // @parm [IN] - Unique client driver id string
    LPCWSTR szReserved,                     // @parm [IN] - Should be set to 0 to ensure compatibility
                                            //              with future versions of Windows CE
    LPCUSB_DRIVER_SETTINGS lpDriverSettings)// @parm [IN] - Specifies how driver is to be loaded
{
    UnusedParameter(szReserved);

    BOOL fRet = FALSE;
    WCHAR szTemp[gcMaxDriverString];
    DWORD dwGarbage;

    HKEY hkUsb;

    if(RegCreateKeyEx(HKEY_LOCAL_MACHINE, gcszUsbRegKey, 0, NULL, 0, 0, NULL,
            &hkUsb, &dwGarbage) == ERROR_SUCCESS)
    {
        HKEY hkLoadOrder;

        if(RegCreateKeyEx(hkUsb, gcszLoadClients, 0, NULL, 0, 0, NULL,
                &hkLoadOrder, &dwGarbage) == ERROR_SUCCESS)
        {
            HKEY hkGroup1;

            GetSettingString(szTemp, gcMaxDriverString, lpDriverSettings->dwVendorId,
                    lpDriverSettings->dwProductId,
                    lpDriverSettings->dwReleaseNumber);

            if(RegCreateKeyEx(hkLoadOrder, szTemp, 0, NULL, 0, 0, NULL,
                    &hkGroup1, &dwGarbage) == ERROR_SUCCESS)
            {
                HKEY hkGroup2;

                GetSettingString(szTemp, gcMaxDriverString, lpDriverSettings->dwDeviceClass,
                        lpDriverSettings->dwDeviceSubClass,
                        lpDriverSettings->dwDeviceProtocol);

                if(RegCreateKeyEx(hkGroup1, szTemp, 0, NULL, 0, 0, NULL,
                        &hkGroup2, &dwGarbage) == ERROR_SUCCESS)
                {
                    HKEY hkGroup3;

                    GetSettingString(szTemp, gcMaxDriverString, lpDriverSettings->dwInterfaceClass,
                            lpDriverSettings->dwInterfaceSubClass,
                            lpDriverSettings->dwInterfaceProtocol);

                    if(RegCreateKeyEx(hkGroup2, szTemp, 0, NULL, 0, 0, NULL,
                            &hkGroup3, &dwGarbage) == ERROR_SUCCESS)
                    {
                        HKEY hkSettings;

                        if(RegCreateKeyEx(hkGroup3, szUniqueDriverId, 0, NULL,
                                0, 0, NULL, &hkSettings, &dwGarbage)
                                == ERROR_SUCCESS)
                        {
                            if(RegSetValueEx(hkSettings, gcszDllName, 0,
                                    REG_SZ, (LPBYTE)szDriverLibFile,
                                    (lstrlen(szDriverLibFile) + 1) *
                                    sizeof(szDriverLibFile[0]))
                                    == ERROR_SUCCESS)
                            {
                                fRet = TRUE;
                            }

                            RegCloseKey(hkSettings);
                        }

                        RegCloseKey(hkGroup3);
                    }

                    RegCloseKey(hkGroup2);
                }

                RegCloseKey(hkGroup1);
            }
            RegCloseKey(hkLoadOrder);
        }
        RegCloseKey(hkUsb);
    }
    return fRet;
}


/*
 * @func   BOOL | UnRegisterClientSettings | Deregister client driver settings.
 * @rdesc  Returns TRUE if settings were successfully removed, otherwise FALSE.
 * @comm   This function should be called by client drivers in their installation routine.
 *         It sets up appropriate registry keys based on lpDriverSettings param so that the
 *         driver will be loaded whenever the device is attached.
 * @xref   <t USB_DRIVER_SETTINGS>  <f RegisterClientSettings>
 */
extern "C" BOOL
UnRegisterClientSettings(
    LPCWSTR szUniqueDriverId,                // @parm [IN] - Client driver DLL name
    LPCWSTR szReserved,                      // @parm [IN] - Should be set to 0 to ensure compatibility
                                             //              with future versions of Windows CE
    LPCUSB_DRIVER_SETTINGS lpDriverSettings) // @parm [IN] - Must match the settings the client was registered with
{
    UnusedParameter(szReserved);
    // REVIEW this procedure has not been tested!

    BOOL fRet = FALSE;
    WCHAR szTemp[gcMaxDriverString];

    LONG lEnum;
    DWORD cchEnum;

    HKEY hkUsb;

    if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, gcszUsbRegKey, 0, 0, &hkUsb)
            == ERROR_SUCCESS)
    {
        HKEY hkLoadOrder;

        if(RegOpenKeyEx(hkUsb, gcszLoadClients, 0, 0, &hkLoadOrder)
                == ERROR_SUCCESS)
        {
            HKEY hkGroup1;

            GetSettingString(szTemp, gcMaxDriverString, 
                    lpDriverSettings->dwVendorId,
                    lpDriverSettings->dwProductId,
                    lpDriverSettings->dwReleaseNumber);

            if(RegOpenKeyEx(hkLoadOrder, szTemp, 0, 0, &hkGroup1)
                    == ERROR_SUCCESS)
            {
                HKEY hkGroup2;

                GetSettingString(szTemp, gcMaxDriverString, 
                        lpDriverSettings->dwDeviceClass,
                        lpDriverSettings->dwDeviceSubClass,
                        lpDriverSettings->dwDeviceProtocol);

                if(RegOpenKeyEx(hkGroup1, szTemp, 0, 0, &hkGroup2)
                        == ERROR_SUCCESS)
                {
                    HKEY hkGroup3;

                    GetSettingString(szTemp, gcMaxDriverString, 
                            lpDriverSettings->dwInterfaceClass,
                            lpDriverSettings->dwInterfaceSubClass,
                            lpDriverSettings->dwInterfaceProtocol);

                    if(RegOpenKeyEx(hkGroup2, szTemp, 0, 0, &hkGroup3)
                            == ERROR_SUCCESS)
                    {
                        if(RegDeleteKey(hkGroup3, szUniqueDriverId)
                                == ERROR_SUCCESS)
                        {
                                fRet = TRUE;
                        }
                        cchEnum = sizeof(szTemp) / sizeof(szTemp[0]);
                        lEnum = RegEnumKeyEx(hkGroup3, 0, szTemp,
                                &cchEnum, NULL, NULL, NULL, NULL);

                        RegCloseKey(hkGroup3);

                        if(lEnum == ERROR_NO_MORE_ITEMS)
                        {
                            GetSettingString(szTemp, gcMaxDriverString,
                                    lpDriverSettings->dwInterfaceClass,
                                    lpDriverSettings->dwInterfaceSubClass,
                                    lpDriverSettings->dwInterfaceProtocol);

                            RegDeleteKey(hkGroup2, szTemp);
                        }
                    }
                    cchEnum = sizeof(szTemp) / sizeof(szTemp[0]);
                    lEnum = RegEnumKeyEx(hkGroup2, 0, szTemp,
                            &cchEnum, NULL, NULL, NULL, NULL);

                    RegCloseKey(hkGroup2);

                    if(lEnum == ERROR_NO_MORE_ITEMS)
                    {
                        GetSettingString(szTemp, gcMaxDriverString,
                                lpDriverSettings->dwDeviceClass,
                                lpDriverSettings->dwDeviceSubClass,
                                lpDriverSettings->dwDeviceProtocol);

                        RegDeleteKey(hkGroup1, szTemp);
                    }
                }

                cchEnum = sizeof(szTemp) / sizeof(szTemp[0]);
                lEnum = RegEnumKeyEx(hkGroup1, 0, szTemp,
                        &cchEnum, NULL, NULL, NULL, NULL);

                RegCloseKey(hkGroup1);

                if(lEnum == ERROR_NO_MORE_ITEMS)
                {
                    GetSettingString(szTemp, gcMaxDriverString, 
                            lpDriverSettings->dwVendorId,
                            lpDriverSettings->dwProductId,
                            lpDriverSettings->dwReleaseNumber);

                    RegDeleteKey(hkLoadOrder, szTemp);
                }
            }
            RegCloseKey(hkLoadOrder);
        }
        RegCloseKey(hkUsb);
    }
    return fRet;
}


